'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('node:fs');
var path = require('node:path');
var tComm = require('t-comm');
var shared = require('@plugin-light/shared');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);

const MAIN = 'MAIN_PACKAGE';
function getSubPackageOfComponentOrPage(componentOrPage, subPackageMap) {
    const subPackages = Object.keys(subPackageMap);
    for (const subPackage of subPackages) {
        if (subPackage === MAIN) {
            continue;
        }
        if (componentOrPage.startsWith(`${subPackage}/`)) {
            return subPackage;
        }
    }
    return MAIN;
}
const checkPlugin = (componentName) => componentName.startsWith('plugin://');

function getPages(bundle) {
    const data = tComm.readJson(bundle['app.json'].source.toString(), 'app.json');
    const { pages = [], subPackages = [], usingComponents = {} } = data;
    const subPackageMap = {
        [MAIN]: pages,
    };
    const parsedSubPackages = subPackages.reduce((acc, subPackage) => {
        const { root, pages = [] } = subPackage;
        const list = pages.map((page) => [root, page].join('/'));
        acc[root] = list;
        return acc;
    }, subPackageMap);
    const reversedSubPackageMap = Object.keys(parsedSubPackages).reduce((acc, subPackage) => {
        const pagesList = parsedSubPackages[subPackage];
        pagesList.forEach((page) => {
            acc[page] = subPackage;
        });
        return acc;
    }, {});
    return {
        pages,
        subPackages,
        parsedSubPackages,
        reversedSubPackageMap,
        globalComponents: usingComponents,
    };
}
const getComponentName = ({ file, files, htmlPostfix }) => {
    if (file.endsWith('.js')) {
        const xml = file.replace(/\.js$/, htmlPostfix);
        if (files.includes(xml)) {
            return file.replace(/\.js$/, '');
        }
    }
    return '';
};
function analyzeComponent(bundle, htmlPostfix) {
    const rawComponentMap = {};
    const files = Object.keys(bundle);
    files.forEach((file) => {
        const componentName = getComponentName({
            file,
            files,
            htmlPostfix,
        });
        if (componentName && !rawComponentMap[componentName]) {
            const jsonName = `${componentName}.json`;
            const json = bundle[jsonName];
            if (!json) {
                console.log(`>>> dispatch vue can't find json: ${jsonName}`);
            }
            else {
                const jsonContent = tComm.readJson(json.source.toString(), jsonName);
                const { usingComponents = {} } = jsonContent;
                const parsed = Object.keys(usingComponents).reduce((acc, name) => {
                    if (checkPlugin(usingComponents[name])) {
                        console.log(`>>> dispatch vue found plugin, file: ${file}, name: ${name}, plugin: ${usingComponents[name]}`);
                    }
                    else {
                        acc[name] = usingComponents[name];
                    }
                    return acc;
                }, {});
                rawComponentMap[componentName] = parsed;
            }
        }
    });
    return rawComponentMap;
}
function formatComponentMap(componentMap = {}) {
    const result = Object.keys(componentMap).reduce((acc, filePath) => {
        const parsed = innerFormatComponentMap(filePath, componentMap[filePath]);
        acc[filePath] = parsed;
        return acc;
    }, {});
    return result;
}
function innerFormatComponentMap(file, usingComponents = {}) {
    const result = Object.keys(usingComponents).reduce((acc, componentName) => {
        const parsed = tComm.parseComponentPath(file, usingComponents[componentName]);
        acc[parsed] = usingComponents[componentName];
        return acc;
    }, {});
    return result;
}
function getComponentMapList(componentMap) {
    const list = Object.keys(componentMap);
    const result = {};
    list.forEach((component) => {
        const usingComponents = componentMap[component];
        const values = Object.keys(usingComponents);
        result[component] = values;
    });
    return result;
}
function getReversedComponentMapList(componentMap) {
    const list = Object.keys(componentMap);
    const result = {};
    list.forEach((componentOrPage) => {
        const usingComponents = componentMap[componentOrPage];
        usingComponents.forEach((inner) => {
            if (!result[inner]) {
                result[inner] = [componentOrPage];
            }
            else if (!result[inner].includes(componentOrPage)) {
                result[inner].push(componentOrPage);
            }
        });
    });
    return result;
}
function getReversedComponentSubPackages(componentMap, reversedSubPackageMap) {
    const list = Object.keys(componentMap);
    const result = {};
    list.forEach((component) => {
        const usingComponents = componentMap[component];
        usingComponents.forEach((inner) => {
            const subPackage = getComponentSubPackage(inner, reversedSubPackageMap);
            if (subPackage) {
                if (!result[component]) {
                    result[component] = [subPackage];
                }
                else if (!result[component].includes(subPackage)) {
                    result[component].push(subPackage);
                }
            }
        });
    });
    return result;
}
function getComponentSubPackage(componentPath, reversedSubPackageMap) {
    const pages = Object.keys(reversedSubPackageMap);
    for (const page of pages) {
        if (componentPath.startsWith(page)) {
            return reversedSubPackageMap[page];
        }
    }
    return '';
}
function getComponentSubPackageFromRoot(componentPath, reversedSubPackageMap) {
    const subPackages = Object.values(reversedSubPackageMap);
    for (let subPackage of subPackages) {
        if (!subPackage.endsWith('/')) {
            subPackage += '/';
        }
        if (componentPath.startsWith(subPackage)) {
            return subPackage.replace(/\/$/, '');
        }
    }
    return '';
}
function findShouldHandleComponents({ componentMap, reversedSubPackageMap, limit, blackList, }) {
    const list = Object.keys(componentMap);
    const result = list.filter((component) => {
        const subPackages = componentMap[component];
        if (subPackages.includes(MAIN)) {
            return false;
        }
        if (blackList && shared.isInBlackList(component, blackList)) {
            return false;
        }
        if (limit) {
            return subPackages.length <= limit;
        }
        return true;
    }).reduce((acc, component) => {
        // 如果组件自身已经在分包中了，则应该排除
        const subPackage = getComponentSubPackageFromRoot(component, reversedSubPackageMap);
        const subs = componentMap[component].filter(sub => sub !== subPackage);
        if (subs.length) {
            acc[component] = subs;
        }
        return acc;
    }, {});
    return result;
}

let isFixed = false;
const TEMP_SYMBOL = '&&&&&&TEMPLATE_STRING&&&&&&&';
const toDeleteFiles = [];
const toModifyFiles = {};
function dispatchVueVitePlugin(options) {
    return {
        name: 'dispatch-vue-vite-plugin',
        enforce: 'post',
        generateBundle(_, bundle) {
            const { emitFile } = this;
            if (isFixed) {
                return;
            }
            console.log('>>> vite dispatch vue start');
            isFixed = true;
            const keys = Object.keys(bundle);
            tComm.saveJsonToLogMore(keys, 'vite-dispatch-vue.bundle-keys.json', {
                max: options?.maxLogNumber,
            });
            const limit = options?.limit ?? 1;
            const dispatchDir = options?.dispatchDir ?? `dispatch-vue-${tComm.randomString(8)}`;
            const blackList = options?.blackList;
            const [wxmlPostfix, wxssPostfix,] = shared.getWxmlAndWxssPostfix();
            const { pages, parsedSubPackages, reversedSubPackageMap, globalComponents } = getPages(bundle);
            const rawComponentMap = analyzeComponent(bundle, wxmlPostfix);
            // 全局组件放到主包，这样就不会分发
            if (pages[0]) {
                rawComponentMap[pages[0]] = {
                    ...(rawComponentMap[pages[0]] || {}),
                    ...(globalComponents || {}),
                };
            }
            const parsedComponentMap = formatComponentMap(rawComponentMap);
            const parsedComponentMapList = getComponentMapList(parsedComponentMap);
            const toIterativeMap = Object.keys(parsedComponentMapList)
                .reduce((acc, item) => {
                const list = parsedComponentMapList[item];
                list.forEach((it) => {
                    acc[item] = {
                        ...(acc[item] || {}),
                        [it]: {},
                    };
                });
                return acc;
            }, {});
            tComm.genIterativeComponentMap(toIterativeMap);
            const flattenedComponentMap = tComm.flattenUsingComponentMap(toIterativeMap);
            const reversedComponentMap = getReversedComponentMapList(flattenedComponentMap);
            // const reversedComponentMapDeep = getDeepComponentMapList(reversedComponentMap);
            const reversedComponentSubPackages = getReversedComponentSubPackages(reversedComponentMap, reversedSubPackageMap);
            const shouldHandleComponentsMap = findShouldHandleComponents({
                componentMap: reversedComponentSubPackages,
                limit,
                blackList,
                reversedSubPackageMap,
            });
            const shouldHandleComponents = Object.keys(shouldHandleComponentsMap);
            modifyRef({
                shouldHandleComponents,
                reversedComponentMap,
                parsedComponentMap,
                parsedSubPackages,
                bundle,
                wxmlPostfix,
                wxssPostfix,
                dispatchDir,
            });
            const replacedComponentRefMap = dispatchComponent({
                shouldHandleComponents,
                bundle,
                wxmlPostfix,
                wxssPostfix,
                reversedComponentSubPackages,
                emitFile,
                dispatchDir,
            });
            tComm.saveJsonToLogMore(rawComponentMap, 'vite-dispatch-vue.component-map-1-raw.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(parsedComponentMap, 'vite-dispatch-vue.component-map-2-parsed.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(parsedComponentMapList, 'vite-dispatch-vue.component-map-3-list.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(toIterativeMap, 'vite-dispatch-vue.component-map-4-iterative.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(flattenedComponentMap, 'vite-dispatch-vue.component-map-5-flatten.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(reversedComponentMap, 'vite-dispatch-vue.component-map-6-reversed.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(reversedComponentSubPackages, 'vite-dispatch-vue.component-map-7-sub-packages.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(shouldHandleComponentsMap, 'vite-dispatch-vue.should-handle-components.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(reversedSubPackageMap, 'vite-dispatch-vue.sub-package-map-2-reversed.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(parsedSubPackages, 'vite-dispatch-vue.sub-package-map-1-parsed.json', {
                max: options?.maxLogNumber,
            });
            tComm.saveJsonToLogMore(replacedComponentRefMap, 'vite-dispatch-vue.replace-component-ref-record.json', {
                max: options?.maxLogNumber,
            });
            console.log('>>> vite dispatch vue end');
        },
        writeBundle() {
            const getDistPath = (file) => path__default["default"].resolve(process.env.UNI_OUTPUT_DIR || '', file);
            toDeleteFiles.forEach((file) => {
                const filePath = getDistPath(file);
                if (fs__default["default"].existsSync(filePath)) {
                    fs__default["default"].unlinkSync(filePath);
                }
            });
            tComm.deleteFolderRecursive(process.env.UNI_OUTPUT_DIR || '', {
                deleteFile: false,
                log: false,
            });
            Object.keys(toModifyFiles).forEach((file) => {
                const content = toModifyFiles[file];
                const filePath = getDistPath(file);
                if (fs__default["default"].existsSync(filePath)) {
                    fs__default["default"].writeFileSync(filePath, content, {
                        encoding: 'utf-8',
                    });
                }
            });
        },
    };
}
function dispatchComponent({ shouldHandleComponents, bundle, wxmlPostfix, wxssPostfix, reversedComponentSubPackages, emitFile, dispatchDir, }) {
    const replacedComponentRefMap = {};
    for (const component of shouldHandleComponents) {
        const subPackages = reversedComponentSubPackages[component];
        for (const subPackage of subPackages) {
            const newDir = `${subPackage}/${dispatchDir}`;
            const dispatchedComponentNewPath = `${newDir}/${component}`;
            copyComponent({
                dispatchedComponentNewPath,
                dispatchedComponentOriginPath: component,
                wxmlPostfix,
                wxssPostfix,
                bundle,
                emitFile,
                subPackage,
                shouldHandleComponents,
                dispatchDir,
                replacedComponentRefMap,
            });
            deleteComponent({
                dispatchedComponentOriginPath: component,
                bundle,
                wxmlPostfix,
                wxssPostfix,
            });
        }
    }
    return replacedComponentRefMap;
}
function copyComponent({ dispatchedComponentNewPath, dispatchedComponentOriginPath, bundle, wxmlPostfix, wxssPostfix, emitFile, subPackage, shouldHandleComponents, dispatchDir, replacedComponentRefMap, }) {
    addBundle({
        dispatchedComponentNewPath,
        dispatchedComponentOriginPath,
        postfix: '.json',
        bundle,
        emitFile,
        subPackage,
        shouldHandleComponents,
        dispatchDir,
        replacedComponentRefMap,
    });
    addBundle({
        dispatchedComponentNewPath,
        dispatchedComponentOriginPath,
        postfix: '.js',
        bundle,
        emitFile,
        subPackage,
        shouldHandleComponents,
        dispatchDir,
        replacedComponentRefMap,
    });
    addBundle({
        dispatchedComponentNewPath,
        dispatchedComponentOriginPath,
        postfix: wxmlPostfix,
        bundle,
        emitFile,
        subPackage,
        shouldHandleComponents,
        dispatchDir,
        replacedComponentRefMap,
    });
    addBundle({
        dispatchedComponentNewPath,
        dispatchedComponentOriginPath,
        postfix: wxssPostfix,
        bundle,
        emitFile,
        subPackage,
        shouldHandleComponents,
        dispatchDir,
        replacedComponentRefMap,
    });
}
function addBundle({ dispatchedComponentNewPath, dispatchedComponentOriginPath, postfix, bundle, emitFile, subPackage, shouldHandleComponents, dispatchDir, replacedComponentRefMap, }) {
    const origin = `${dispatchedComponentOriginPath}${postfix}`;
    if (!bundle[origin]) {
        console.warn(`not found ${origin}`);
        return;
    }
    let source = (bundle[origin].source || bundle[origin].code);
    if (!source) {
        source = '';
    }
    source = replaceFilePathByReg({
        source,
        dispatchedComponentNewPath,
        dispatchedComponentOriginPath,
        subPackage,
        shouldHandleComponents,
        dispatchDir,
        replacedComponentRefMap,
    });
    source = source.toString();
    emitFile({
        type: 'asset',
        fileName: `${dispatchedComponentNewPath}${postfix}`,
        source,
    });
}
function modifyRef({ shouldHandleComponents, reversedComponentMap, parsedComponentMap, parsedSubPackages, bundle, wxmlPostfix, wxssPostfix, dispatchDir, }) {
    for (const component of shouldHandleComponents) {
        const componentsOrPages = reversedComponentMap[component];
        for (const componentOrPage of componentsOrPages) {
            if (shouldHandleComponents.includes(componentOrPage))
                continue;
            // 之前的组件引入路径，如 ../../xxx
            const componentImportation = parsedComponentMap[componentOrPage][component];
            const subPackage = getSubPackageOfComponentOrPage(componentOrPage, parsedSubPackages);
            const newImportation = [subPackage, dispatchDir, component].join('/');
            innerModifyRef({
                bundle,
                componentOrPage,
                componentImportation,
                newImportation,
                postfix: '.json',
                record: false,
            });
            innerModifyRef({
                bundle,
                componentOrPage,
                componentImportation,
                newImportation,
                postfix: '.js',
                record: true,
            });
            innerModifyRef({
                bundle,
                componentOrPage,
                componentImportation,
                newImportation,
                postfix: wxmlPostfix,
            });
            innerModifyRef({
                bundle,
                componentOrPage,
                componentImportation,
                newImportation,
                postfix: wxssPostfix,
                record: true,
            });
        }
    }
}
function innerModifyRef({ bundle, componentOrPage, componentImportation, newImportation, postfix, record = false, }) {
    const fileName = `${componentOrPage}${postfix}`;
    const file = bundle[fileName];
    if (!file) {
        console.error(`>>> dispatch vue error: not found ${fileName}`);
    }
    let key = 'source';
    if (postfix === '.js') {
        key = 'code';
    }
    const source = file[key].toString();
    const relativePath = tComm.getRelativePath(componentOrPage, newImportation)
        .split(path__default["default"].sep)
        .join('/');
    file[key] = source.replaceAll(componentImportation, relativePath);
    if (record) {
        toModifyFiles[fileName] = file[key];
    }
}
function deleteComponent({ dispatchedComponentOriginPath, 
// bundle,
wxmlPostfix, wxssPostfix, }) {
    // js 文件和 wxss 文件，无法通过 generateBundle 钩子删除
    toDeleteFiles.push(`${dispatchedComponentOriginPath}.js`);
    toDeleteFiles.push(`${dispatchedComponentOriginPath}${wxssPostfix}`);
    // json 文件也不从 generateBundle 钩子中删除，防止复制（emitFile）的时候找不到
    toDeleteFiles.push(`${dispatchedComponentOriginPath}.json`);
    toDeleteFiles.push(`${dispatchedComponentOriginPath}${wxmlPostfix}`);
    // delete bundle[`${dispatchedComponentOriginPath}.json`];
    // delete bundle[`${dispatchedComponentOriginPath}${wxmlPostfix}`];
}
function replaceFilePathByReg({ source, dispatchedComponentOriginPath, dispatchedComponentNewPath, subPackage, shouldHandleComponents, dispatchDir, replacedComponentRefMap, }) {
    const reg = /('|")(\.\.?\/[/\w-.@]+)(?:'|")/g;
    let newSource = source;
    let match = reg.exec(newSource);
    while (match) {
        const { 0: origin, 1: quoteSymbol, 2: pre } = match;
        const target = genTarget({
            originImportation: pre,
            dispatchedComponentOriginPath,
            dispatchedComponentNewPath,
            subPackage,
            shouldHandleComponents,
            dispatchDir,
        });
        const newContent = `${quoteSymbol}${TEMP_SYMBOL}${target}${TEMP_SYMBOL}${quoteSymbol}`;
        newSource = newSource.replace(origin, newContent);
        replacedComponentRefMap[dispatchedComponentOriginPath] = [
            ...(replacedComponentRefMap[dispatchedComponentOriginPath] || []),
            { [origin]: newContent },
        ];
        match = reg.exec(newSource);
    }
    newSource = newSource.replace(new RegExp(TEMP_SYMBOL, 'g'), '');
    return newSource;
}
function genTarget({ originImportation, dispatchedComponentOriginPath, dispatchedComponentNewPath, subPackage, shouldHandleComponents, dispatchDir, }) {
    let filePath = tComm.parseComponentPath(dispatchedComponentOriginPath, originImportation);
    if (shouldHandleComponents.includes(filePath)) {
        filePath = [subPackage, dispatchDir, filePath].join('/');
    }
    const relativePath = tComm.getRelativePath(dispatchedComponentNewPath, filePath);
    return relativePath
        .split(path__default["default"].sep)
        .join('/');
}

exports.dispatchVueVitePlugin = dispatchVueVitePlugin;
